/**************************************************************************
 **   FILE NAME		: ifx_os.h
 **   PROJECT		: OS Interface
 **   MODULES	      	: Common OS layer for Infineon Modules
 **   SRC VERSION	: V1.0
 **
 **   DATE				  	: 2-11-2004
 **   AUTHOR			  	: Hari
 **   DESCRIPTION			: This file contains common OS wrappers and C 
 **                       library functions.	
 **   FUNCTIONS			:	
 **   COMPILER				:
 **   REFERENCE			:
 **							
 **   COPYRIGHT			: Infineon Technologies AG 2003-2004

 **  Version Control Section  **
 **   $Author$
 **   $Date$
 **   $Revisions$
 **   $Log$       Revision history
***********************************************************************/

#ifndef __IFX_OS_H__
#define __IFX_OS_H__

#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <features.h>
#include <ctype.h>
#include <netdb.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#define IFX_OS_PROTOCOL_TCP 1
#define IFX_OS_PROTOCOL_UDP 2
#define IFX_OS_TCP_SOCKTYPE_CLIENT 1
#define IFX_OS_TCP_SOCKTYPE_SERVER 2

/********** Fifo Operations **********/

EXTERN int32 IFX_OS_CreateFifo(IN uchar8 * pucName);

#define IFX_OS_RemoveFifo(pucName, Fd) \
{ \
   close(Fd); \
   unlink(pucName); \
}

EXTERN int32 IFX_OS_OpenFifo(IN uchar8 * pucName, IN int32 iFlags);

#define IFX_OS_CloseFifo(Fd) close(Fd)

#define IFX_OS_ReadFifo(iFd, pcRead, iSize) read(iFd, pcRead, iSize)

#define IFX_OS_WriteFifo(iFd, pcWrite, iSize) write(iFd, pcWrite, iSize)

#define IFX_OS_Read(iFd, pcRead, iSize) read(iFd, pcRead, iSize)

#define IFX_OS_Write(iFd, pcWrite, iSize) write(iFd, pcWrite, iSize)

#define IFX_OS_Close(Fd) close(Fd)

/********** Memory Operations **********/

#define IFX_OS_Memset(pvDst, iChar, uiSize) \
{ \
   uchar8 * pucPtr = (uchar8 *) pvDst; \
   while (uiSize--) \
   { \
      *pucPtr++ = (uchar8) iChar; \
   } \
}

#define IFX_OS_Memcpy(pvDst, pvSrc, uiSize) \
{ \
   uchar8 * pucDst1 = (uchar8 *) pvDst ; \
   uchar8 * pucSrc1 = (uchar8 *) pvSrc ; \
   while (uiSize--) \
   { \
      *pucDst1++ = *pucSrc1++; \
   } \
}

EXTERN int32 IFX_OS_Memcmp(IN const void * pPtr1, IN const void * pPtr2,
              IN uint32 uiSize);

/********** String Operations **********/

EXTERN int32 IFX_OS_Strlen(IN const char8 * pcPtr);

EXTERN char8 * IFX_OS_Strcpy(IN char8 * pcDst, IN const char8 * pcSrc);

/************** File Operations ****************/

#define IFX_OS_OpenFile(pcFileName, iFlag) open(pcFileName, iFlag)

#define IFX_OS_CloseFile(Fd) close(Fd)

#define IFX_OS_DeleteFile(pcName, Fd) \
{ \
   close(Fd); \
   unlink(pcName); \
}

#define IFX_OS_ReadFile(iFd, pcRead, iSize) read(iFd, pcRead, iSize)

#define IFX_OS_WriteFile(iFd, pcWrite, iSize) write(iFd, pcWrite, iSize)

#define IFX_OS_Lseek(Fd, OffSet, Whence) lseek(Fd, OffSet, Whence)

/*************** Memory Operations ***************/

#define IFX_OS_Malloc(iBytes) malloc(iBytes)
#define IFX_OS_Free(pBuff) free(pBuff)

#ifdef COMPILE

#endif


/**********************System V Shared Memory **************/

/* Create a shared memory*/
#define IFX_OS_ShmCreate(iKey,iSize,iFlag) shmget(iKey,iSize,iFlag)

/* Attach shared memory to the process*/
#define IFX_OS_ShmAttach(iShmId,pcShmAddr,iFlag) shmat(iShmId,pcShmAddr,iFlag)

/*Detach the shared memory form the process*/
#define IFX_OS_ShmDetach(pcShmAddr) shmdt(pcShmAddr)

/* Control operations on Shared memory*/
#define IFX_OS_ShmCtl(iShmId, iCommand, pxShmidDs) shmctl(iShmId, iCommand, pxShmidDs)
/**********************System V Semphore **********************/

/* Create a named Semaphore */
#define IFX_OS_SemCreate(iKey,iNSems,iFlag) semget(iKey,iNSems,iFlag)

/* Control semphore operations*/
#define IFX_OS_SemCtl(iSemId,iSemNum,iCmd,arg ) semctl(iSemId,iSemNum,iCmd,arg )

/* Remove semaphore */
#define IFX_OS_RemoveSem(iSemId,iSemNum) semctl(iSemId,iSemNum,IPC_RMID)

/* Sempaphore operations*/
#define IFX_OS_SemOp(iSemId,pxSemBuff,iNoSems) semop(iSemId,pxSemBuff,iNoSems)

/* Lock Semphore*/
EXTERN int32 IFX_OS_LOCK(IN int32 iSemId);

/*Unlock Semphore*/
EXTERN int32 IFX_OS_UNLOCK(IN int32 iSemId);

/* Sempahore argument passing union */
typedef union
{
   int32 val;
   struct semid_ds *buff;
   uint16 *array;
 } ux_IFX_OS_SEMUN;


/******************System V Utility Functions****************/

/* Function used in generating Keys from Path names */
#define IFX_OS_FTOK(pcName,iProjId) ftok(pcName,iProjId)


#endif /* __IFX_OS_H__ */
