/**************************************************************************
**   FILE NAME		: ifx_lldp_tx_module.c
**   PROJECT		: Inca-Ip
**   MODULES	    : LLDP
**   SRC VERSION	: V1.0
**
**   DATE			:12-04-2007
**   AUTHOR			:Pavan GM
**   DESCRIPTION	:Source file 	
**   COMPILER		:GCC
**   REFERENCE		:Coding Guideline Version 2.0
**							
**   COPYRIGHT      :Copyright © 2004
**                   Infineon Technologies AG
**                   St. Martin Strasse 53; 81669 München, Germany
**   DESCLAIMER     :Any use of this Software is subject to the conclusion 
**                   of a respective License Agreement.
**                   Without such a License Agreement no rights to the 
**         			 Software are granted. 
**
**  Version Control Section 
**   $Author$
**   $Date$
**   $Revisions$
**   $Log$       Revision history
***********************************************************************/

#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#ifdef DEBUG_ENABLED
#include <stdio.h>
#endif

#include <signal.h>

#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_lldp_tx_sm.h"
#include "ifx_lldp_rx_sm.h"
#include "ifx_lldp_mib_if.h"
#include "ifx_lldp_cm.h"
#include "ifx_lldp_tlv.h"
#include "ifx_lldp_rx_module.h"
#include "ifx_lldp_tx_module.h"
#include "ifx_lldp_port.h"
#include "ifx_lldp_timer_if.h"


/* External variables */
EXTERN struct lldp_port *vpxlldp_port;

/******************************************************************
*  Function Name    : txInitializeLLDP
*  Description      : This function initializes the transmit module
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

uint8 txInitializeLLDP(struct lldp_port *lldp_port)
{
  PRINT("\nIN->txInitializeLLDP\n");

  /*get local MIB cfg data and update lldp_port->adminstatus.portenabled would
   *be updated by main once skt initialised.
   *These are checked in the FSM for proceeding to the next state*/

  struct LldpConfigMib ConfigMib;
  
    PRINT("\ntxInitializeLLDP:calling getMibData for lldp_port->uiPortIndex = %d \n", lldp_port->uiPortIndex );
  if(getMibData(localConfig,lldp_port->uiPortIndex ,NULL,(void*)&ConfigMib) ==
     MIB_SUCCESS){
    PRINT("\ntxInitializeLLDP:got config mib data \n");
	}else{
    PRINT("\ntxInitializeLLDP:FAILED to get config mib data!!! \n");
	}

  lldp_port->adminStatus =  ConfigMib.adminStatus;
  PRINT("\nOUT->txInitializeLLDP lldp_port->adminStatus = %d \n", lldp_port->adminStatus );

  PRINT("\nOUT->txInitializeLLDP\n");
  
  return 0;
}


/******************************************************************
*  Function Name    : mibConstrInfoLLDPDU
*  Description      : This function creates an Info LLDPDU
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void mibConstrInfoLLDPDU(struct lldp_port *lldp_port)
{
  PRINT("\nIN->mibConstrInfoLLDPDU\n");
  // This code will only work for Linux!
  struct lldp_tlv_list *tlv_list = NULL;
  uint32 frame_offset=0;
  uint8 temp=0;

  ConstrLLDPheader(lldp_port,&frame_offset); 
  PRINT("\nframe size after fillin header =%d\n",frame_offset);
  
  /*get local mib data and supported TLV listXXX*/
  struct LldpMib LocalMib;
  struct LldpTlvSelectionMib lldptlv;
  if(getMibData(localMib,lldp_port->uiPortIndex ,NULL,(void*)&LocalMib) 
     == MIB_SUCCESS)
    PRINT("mibConstrInfoLLDPDU:got local mib data\n");
    
  if(getMibData(localTlvSel,lldp_port->uiPortIndex ,NULL,(void*)&lldptlv) ==
     MIB_SUCCESS)
    PRINT("mibConstrInfoLLDPDU:got supp tlvs\n");
  
  /**********append mandatory tlvs now************/
  // CHASSIS ID TLV *MUST* be first.
  add_tlv(create_chassis_id_tlv(&LocalMib.chassis_id), &tlv_list);

  add_tlv(create_port_id_tlv(&LocalMib.port_id), &tlv_list);

  // TTL TLV *MUST* be third.
  add_tlv(create_ttl_tlv(lldp_port->tx.txTTL), &tlv_list);

  /*append optional tlvs now if supported. */
	
  if(lldptlv.portConfigTlvTxEnable & SYSTEM_CAPABILITIES)
     add_tlv(create_system_capabilities_tlv(&LocalMib.sys_cap), &tlv_list);

 /*if sys cap there fine else ck if any other is enabled to fit the 4th pos*/

  /*MAC/PHY should be fifth*/
  if(lldptlv.portConfigTlvTxEnable & MAC_PHY_CONFIGURATION_STATUS)
  	add_tlv(create_macPhy_configStatus_tlv(&LocalMib.mac_phy), &tlv_list);

  if(lldptlv.portConfigTlvTxEnable & MED_CAPABILITIES)
	{
		if(LocalMib.med_cap.capSupported & 0x01)
  		add_tlv(create_medCapabilities_tlv(&LocalMib.med_cap), &tlv_list);

    /* add more than 1 n/w policy*/
    if(lldptlv.portConfigTlvTxEnable & NETWORK_POLICY)
    {
      if(LocalMib.med_cap.capSupported & 0x02)
	    {
	      for(temp=0;temp<LocalMib.network_policy.noOfNetworkPolicyTlvs;temp++)
	  	    add_tlv(create_networkPolicy_tlv(
	              &LocalMib.network_policy.network_policy_array[temp]), &tlv_list);
	    }
	  }

	  if(lldptlv.portConfigTlvTxEnable & EXTENDED_POWER_VIA_MDI)
	  {
	   if((LocalMib.med_cap.capSupported & 0x08) || 
	      (LocalMib.med_cap.capSupported & 0x10)) 
	     	add_tlv(create_extendedPower_tlv(&LocalMib.ext_power), &tlv_list);
	  }

	}
  /*append MED TLV's now*/
  // END OF TLV *MUST* be last.
  add_tlv(create_end_of_lldpdu_tlv(), &tlv_list);

  ConstrLLDPDUinfo(lldp_port,&tlv_list,frame_offset);
}

/******************************************************************
*  Function Name    : ConstrLLDPheader
*  Description      : This function builds LLDP header consisting of
*		      src addr,dest addr and ethertype
*  Input Values     : lldp_port
*  Output Values    : frame offset in the final LLDPDU str
*  Return Value     : None
*  Notes            :
*********************************************************************/


void ConstrLLDPheader(struct lldp_port *lldp_port,uint32 *frameoff)
{
  PRINT("\nIN->ConstrLLDPheader\n");
  struct eth_hdr tx_hdr;
  
 /* As per section 10.3.1, verify the destination and ethertype */
  tx_hdr.dst[0] = 0x01;
  tx_hdr.dst[1] = 0x80;
  tx_hdr.dst[2] = 0xc2;
  tx_hdr.dst[3] = 0x00;
  tx_hdr.dst[4] = 0x00;
  tx_hdr.dst[5] = 0x0e;

  tx_hdr.src[0] = lldp_port->source_mac[0];
  tx_hdr.src[1] = lldp_port->source_mac[1];
  tx_hdr.src[2] = lldp_port->source_mac[2];
  tx_hdr.src[3] = lldp_port->source_mac[3];
  tx_hdr.src[4] = lldp_port->source_mac[4];
  tx_hdr.src[5] = lldp_port->source_mac[5];
  
  tx_hdr.ethertype = htons(0x88cc);

  uint32 frame_offset = 0;

  memcpy(&lldp_port->tx.frame[frame_offset], &tx_hdr, sizeof(tx_hdr));
  frame_offset += sizeof(struct eth_hdr);

  *frameoff=frame_offset;
  PRINT("\nOUT->ConstrLLDPheader\n");
}

/******************************************************************
*  Function Name    : ConstrLLDPDUinfo
*  Description      : This function builds the final LLDPDU str from the 
*		      TLV format
*  Input Values     : lldp_port,double pointer of the TLV list,frame offset
*  Output Values    : updated tlv list
*  Return Value     : None
*  Notes            :
*********************************************************************/
void ConstrLLDPDUinfo(struct lldp_port *lldp_port,
                      struct lldp_tlv_list **tlvlist,
                      uint32 fram_off)
{
  PRINT("\nIN->ConstrLLDPDUinfo\n");
  PRINT("\nframe offset to begin for info =%d\n",fram_off);

  struct lldp_tlv_list *tmp = *tlvlist;
  uint32 frame_offset=fram_off;

  while(tmp != NULL) {
    PRINT("\nFLattening TLV\n");
    struct lldp_flat_tlv *tlv = flatten_tlv(tmp->tlv);

    memcpy(&lldp_port->tx.frame[frame_offset], tlv->tlv, tlv->size);

    frame_offset += tlv->size;

    /*free memory allocated for flat tlv in flatten_tlv*/ 
    free(tlv->tlv);
    free(tlv);

    tmp = tmp->next;
  }
  /*free memory allocated for all tlvs in the TLV list*/
  destroy_tlv_list(tlvlist);
  
  PRINT("\nfinal frame size =%d\n",frame_offset);
  // Pad to 64 bytes
  if(frame_offset < 64) {
    PRINT("\nframe offset less than 64\n");
    lldp_port->tx.sendsize = 64;
  } else {
    PRINT("\nframe offset >= 64\n");
    lldp_port->tx.sendsize = frame_offset;
  }
  PRINT("\nOUT->ConstrLLDPDUinfo\n");
}

/******************************************************************
*  Function Name    : mibConstrShutdownLLDPDU
*  Description      : This function creates an shutdown LLDPDU
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void mibConstrShutdownLLDPDU(struct lldp_port *lldp_port)
{
  struct lldp_tlv_list *tlv_list = NULL;
  uint32 frame_offset=0;

  ConstrLLDPheader(lldp_port,&frame_offset); 
  
  /*get local mib data XXX*/
  struct LldpMib LocalMib;
  if(getMibData(localMib,lldp_port->uiPortIndex ,NULL,(void*)&LocalMib) ==
     MIB_SUCCESS)
    PRINT("\nmibConstrShutdownLLDPDU:got local mib data\n");


  /****append mandatory tlvs now*****/

  // CHASSIS ID TLV *MUST* be first.
  add_tlv(create_chassis_id_tlv(&LocalMib.chassis_id), &tlv_list);

  // PORT ID TLV *MUST* be second.
  add_tlv(create_port_id_tlv(&LocalMib.port_id), &tlv_list);
 //struct lldp_tlv *end_of_lldpdu_tlv = create_end_of_lldpdu_tlv(lldp_port);
  
  // TTL TLV *MUST* be third.Set TTL TLV to zero
  lldp_port->tx.txTTL = 0; 	
  add_tlv(create_ttl_tlv(lldp_port->tx.txTTL), &tlv_list);
 
  /*append optional tlvs now if supported. */
  add_tlv(create_system_capabilities_tlv(&LocalMib.sys_cap), &tlv_list);

  /*MAC/PHY should be fifth*/
  add_tlv(create_macPhy_configStatus_tlv(&LocalMib.mac_phy), &tlv_list);

  // END OF TLV *MUST* be last.
  add_tlv(create_end_of_lldpdu_tlv(), &tlv_list);

  ConstrLLDPDUinfo(lldp_port,&tlv_list,frame_offset);
}

/******************************************************************
*  Function Name    : txFrame
*  Description      : This function handles the LLDP transmission over
*		      the RAW SOCKET interface
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

uint8 txFrame(struct lldp_port *lldp_port)
{
  int status = 0;
  PRINT("\nIN->txFrame\n");

  status = sendto(lldp_port->socket, htons(lldp_port->tx.frame),
                  lldp_port->tx.sendsize, 0, NULL, 0);  

  // Should only need to bzero to sendsize, because we shouldn't have 
  // filled it more than that. ;)
  bzero(&lldp_port->tx.frame[0], lldp_port->tx.sendsize);
  
  // Update the statsFramesOutTotal counter...
  lldp_port->tx.statistics.statsFramesOutTotal++;
  LLdp_tx_UpdateMIBStats(lldp_port);

  /*Reset it back to 0*/
  lldp_port->tx.statistics.statsFramesOutTotal=0;

  return 0;
}

/******************************************************************
*  Function Name    : LLdp_tx_HandleEvent
*  Description      : This function handles the event passed by the LLDP Main
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void LLdp_tx_HandleEvent(uint16 uiPortIndex,x_IFX_LLDP_Lldp_Event *event)
{

  PRINT("\nIN->LLdp_tx_HandleEvent\n");
  struct lldp_port *lldp_port  = vpxlldp_port+uiPortIndex;

  switch(event->eEventType) 
  {
    case LLDP_INIT_MSG :
      
        lldp_port->tx.currstate = -1;
        lldp_port->tx.state = TX_LLDP_INITIALIZE; 
      
    break;
    case LLDP_LOCAL_MIB_CHANGE_MSG :
      lldp_port->tx.somethingChangedLocal = TRUE;
    break;
    case LLDP_CONFIG_MSG :
    break;
    case LLDP_PORT_ENABLED_MSG :
    break;
    case LLDP_PORT_DISABLED_MSG :
    break;
    case LLDP_LOCAL_TIMER_MSG : /*Timeout msg */
    {
      if(event->EventID == TX_LLDP_FRAME_INTERVAL_TIMEOUT) 
         lldp_port->tx.timers.txTTR = 0; 
      else if(event->EventID == TX_LLDP_LOCAL_MIB_CHANGES_TIMEOUT)
        lldp_port->tx.timers.txDelayWhile = 0;
      else if(event->EventID == TX_LLDP_SHUTDOWN_TIMEOUT)
       lldp_port->tx.timers.txShutdownWhile = 0;
      break;
    }
    case LLDP_SHUTDOWN_MSG :  
    {
      struct LldpConfigMib Config;
      getMibData(localConfig,lldp_port->uiPortIndex,NULL,(void *)&Config);
      lldp_port->adminStatus=Config.adminStatus;
      PRINT("\nIN->LLdp_tx_HandleEvent lldp_port->adminStatus = %d\n", lldp_port->adminStatus );

      txStatemachineRun(lldp_port);
      LLdp_tx_cleanup(lldp_port);
    break;
    }
    default:
    return;
  }
  if(event->eEventType != LLDP_SHUTDOWN_MSG )
   /*Invoke TX FSM*/
   txStatemachineRun(lldp_port);
}

/******************************************************************
*  Function Name    : LLdp_tx_UpdateMIBStats
*  Description      : This function updates Tx MIB stats
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void LLdp_tx_UpdateMIBStats(struct lldp_port *lldp_port)
{
   struct LldpStatsMib TxStats;

  /*get current stats*/
  getMibData(Stats,lldp_port->uiPortIndex,NULL,(void *)&TxStats);
  
  (TxStats.statsFramesOutTotal) += 
    (lldp_port->tx.statistics.statsFramesOutTotal);
  if(setMibData(Stats,lldp_port->uiPortIndex, NULL, (void *)(&TxStats)) ==
     MIB_SUCCESS)
    PRINT("\nLLdp_tx_UpdateMIBStats:updated TX stats\n");
}

/*********************************************************************
*  Function Name    : LLdp_tx_cleanup
*  Description      : This function cleans up the tx port before shutdown
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

int32 LLdp_tx_cleanup(struct lldp_port *lldp_port)
{
  int32 ret=LLDP_SUCCESS;
  /* These 2 timers would be stopped in do_shutdown_frame()
  IFX_LLDP_StopTimer(lldp_port->tx.unInterval);  
  IFX_LLDP_StopTimer(lldp_port->tx.unDelay);  */
  IFX_LLDP_StopTimer(lldp_port->tx.unShutDown); 

  return ret; 
}

