/**************************************************************************
 **   FILE NAME       : ifx_lldp_intf.h
 **   PROJECT         : LLDP
 **   MODULES         : LLDP Intf Module
 **   SRC VERSION     : V0.1
 **   DATE            : 17-4-2007
 **   AUTHOR          : Supreet M S, Awanish Gautam, Pavan GM
 **   DESCRIPTION     : 
 **   DEPENDENCIES    : ifx_common_defs.h
 **   FUNCTIONS       :
 **   COMPILER        : gcc
 **   REFERENCE       : Coding guide lines for VSS ,
 **
 **   COPYRIGHT       : Copyright  2004 Infineon Technologies AG
 **                     St. Martin Strasse 53; 81669 Mnchen, Germany
 **                     Any use of this Software is subject to the conclusion
 **                     of a respective License Agreement.Without such a
 **                     License Agreement no rights to the Software are granted.
 **  Version Control Section  **
 **   $Author$
 **   $Date$
 **   $Revisions$
 **   $Log$       Revision history
***********************************************************************/
#ifndef __IFX_LLDP_INTF_H__
#define __IFX_LLDP_INTF_H__

/* Callback function to be defined by LLDP Manager */
typedef void (*lldpMgr_intf_funcptr)(x_IFX_LLDP_LLDPAGENT_MSG*);

/* This enum defines all ports supported on the INCA IP 2 board */
typedef enum
{
  IFX_LLDP_PC = 0,
  IFX_LLDP_LAN = 1
}e_IFX_LLDP_PORT_NAME;

/* This enum defines all types of possible admin status values. Refer to 
   802.1 AB, section 10.5.1 for details */
typedef enum {
  IFX_LLDP_INTF_DISABLED = 0,
  IFX_LLDP_INTF_ENABLED_TX_ONLY ,
  IFX_LLDP_INTF_ENABLED_RX_ONLY,
  IFX_LLDP_INTF_ENABLED_RXTX
}e_IFX_LLDP_INTF_AdminStatus;

/* This structure defines the local LLDP configuration parameters. 
   Refer to Table - 11.1 in 802.1 AB spec */
typedef struct {
  /* 802.1 AB:10.5.1 */
  uint8 ucIntfAdminStatus;
  /* 802.1 AB:10.5.3.3 */
  uint16 unIntfReinitDelay;
  /* 802.1 AB:10.5.3.3 */
  uint16 unIntfMsgTxHold;
  /* 802.1 AB:10.5.3.3 */
  uint16 unIntfMsgTxInterval;
  /* 802.1 AB:10.5.3.3 */
  uint16 unIntfTxDelay;
  /* TIA 1057: 12.1 */
  uint32 uiIntfMedFastStartRepeatCount;
} x_IFX_LLDP_CONFIGCFG;

/* This structure defines all statistical information related to the
   functioning of LLDP Agent. Refer to 802.1 AB, section 10.5.2 for details */
typedef struct {
  uint32 ulPortStatsAgeoutsTotal;
  uint32 ulPortStatsFramesDiscardedTotal;
  uint32 ulPortStatsFramesInErrorsTotal;
  uint32 ulPortStatsFramesInTotal;
  uint32 ulPortStatsFramesOutTotal;
  uint32 ulPortStatsTLVsDiscardedTotal;
  uint32 ulPortStatsTLVsUnrecognizedTotal;
}x_IFX_LLDP_PORT_STATS;

/* This structure defines the System Capabilities TLV. Refer to 
   802.1 AB, section 9.5.8 for details */
typedef struct
{
  boolean bOther;
  boolean bOther_status;
  boolean bRepeater;
  boolean bRepeater_status;
  boolean bBridge;
  boolean bBridge_status;
  boolean bWlanAp;
  boolean bWlanAp_status;
  boolean bRouter;
  boolean bRouter_status;
  boolean bTelephone;
  boolean bTelephone_status;
  boolean bCableDevice;
  boolean bCableDevice_status;
  boolean bStationOnly;
  boolean bStationOnly_status;
}x_IFX_LLDP_SYS_CAPABILITIES;

/* This structure defines the MAC/PHY TLV. Refer to 802.1 AB, section G.2
   for details */
typedef struct
{
  uint8 uiSupport;
  uint8 uiStatus;
  uint16 uiAdvertisedCapabilities;
  uint16 uiMauType;
}x_IFX_LLDP_MAC_PHY_CONFIG_STATUS;

/* This structure defines the MED Capabilities TLV. Refer to TIA 1057, section
   10.2.2 for details */
typedef struct
{
  boolean bCapabilities;
  boolean bNetworkPolicy;
  boolean bLocationIdentification;
  boolean bExtendedPowerMdiPD;
  boolean bExtendedPowerMdiPse;
  boolean bInventory;
  uint8 uiDeviceType;
}x_IFX_LLDP_MED_CAPABILITIES;

enum {
  IFX_LLDP_INTF_CAPABILITIES = 0x01,
  IFX_LLDP_INTF_NETWORKPOLICY = 0x02,
  IFX_LLDP_INTF_LOCIDENTIFICATION = 0x04,
  IFX_LLDP_INTF_EXEPOWERPSE = 0x08,
  IFX_LLDP_INTF_EXEPOWERPD = 0x10,
  IFX_LLDP_INTF_INVENTORY = 0x20
};
/* This enum defines all types of devices such as Network Connectivity Device, 
   Endpoint Device. Refer to TIA 1057, section 6 for details */
typedef enum {
  IFX_LLDP_INTF_NOTDEFINED = 0,
  IFX_LLDP_INTF_CLASSI,
  IFX_LLDP_INTF_CLASSII,
  IFX_LLDP_INTF_CLASSIII,
  IFX_LLDP_INTF_NETWORKCONNECTIVITY
}e_IFX_LLDP_DEVICETYPE;

/* This structure defines the Network policy TLV. Refer to TIA 1057, section 
   10.2.3 for details */
typedef struct
{
  uint8 uiApplicationType;
  boolean bUnknownFlag;
  boolean bTaggedFlag;
  uint16 uiVlanId;
  uint8 uiL2Priority;
  uint8 uiDscpValue;
}x_IFX_LLDP_NETWORK_POLICY_TLV;

typedef struct
{
  uint8 uiNoOfNetworkPolicyTlvs;
  x_IFX_LLDP_NETWORK_POLICY_TLV xNetworkPolicyArray[IFX_LLDP_MAXNETWORKTLVS];
}x_IFX_LLDP_NETWORK_POLICY;

/* This structure defines Extended Power-Via-MDI TLV. Refer to TIA 1057, 
   section 10.2.5 for details */ 
typedef struct
{
  uint8 uiType;
  uint8 uiSource;
  uint8 uiPriority;
  uint16 uiValue;
}x_IFX_LLDP_EXTENDED_POWER;

/* This structure stores the Port VLAN ID TLV related information. Refer to 
   TIA 1057, section 5.3 and 802.1 AB, section E.2.1 for details */
typedef struct {
	uint16 uiPortVlanId;
}x_IFX_LLDP_PORT_VLAN_ID;

/* This structure stores the Port And Protocol VLAN ID TLV related information.
   Refer to TIA 1057, section 5.3 and 802.1 AB, section E.2.2 for details */
typedef struct {
  uint8 uiProtoVlanSupported;
  uint8 uiProtoVlanEnabled;
	uint16 uiProtoVlanId;
}x_IFX_LLDP_PORT_AND_PROTOCOL_VLAN_ID;

/* This structure stores the VLAN Name TLV related information. Refer to 
   TIA 1057, section 5.3 and 802.1 AB, section E.2.3 for details */
typedef struct {
  uint16 uiVlanId;
  uint8 uiVlanNameLength;
  uint8 uiaVlanName[32];
}x_IFX_LLDP_VLAN_NAME;

/* This structure stores the Protocol Identity TLV related information.
    Refer to TIA 1057, section 5.3 and 802.1 AB,section E.2.3 for details */
typedef struct {
  uint8 uiProtoIdLength;
  uint8 uiProtoId[255];
}x_IFX_LLDP_PROTOCOL_IDENTITY;

/* This enum encompasses all possible TLV types defined in 802.1 AB 
   and TIA 1057 */
typedef enum
{
  IFX_LLDP_INTF_CHASSIS_ID = 1,
  IFX_LLDP_INTF_PORT_ID = 2,
  IFX_LLDP_INTF_SYSTEM_CAPABILITIES = 4,
  IFX_LLDP_INTF_TIME_TO_LIVE =8,
  IFX_LLDP_INTF_PORT_VLAN_ID = 0x10,
  IFX_LLDP_INTF_PORT_AND_PROTOCOL_VLAN_ID = 0x20,
  IFX_LLDP_INTF_VLAN_NAME = 0x40,
  IFX_LLDP_INTF_PROTOCOL_IDENTITY =0x80,
  IFX_LLDP_INTF_MAC_PHY_CONFIGURATION_STATUS = 0x100,
  IFX_LLDP_INTF_MED_CAPABILITIES = 0x200,
  IFX_LLDP_INTF_NETWORK_POLICY = 0x400,
  IFX_LLDP_INTF_EXTENDED_POWER_VIA_MDI = 0x800,
  IFX_LLDP_INTF_END_OF_LLDPDU =0x1000
}e_IFX_LLDP_INTF_TLV;

/* This union is used to fetch local TLV information from the LLDP Agent one at 
   a time */
typedef union
{
  uchar8 ucChassisId[255];
  uchar8 ucPortId[255];
  x_IFX_LLDP_SYS_CAPABILITIES xSystemCapabilities;
  x_IFX_LLDP_PORT_VLAN_ID xPortVlanId;
  x_IFX_LLDP_PORT_AND_PROTOCOL_VLAN_ID xPortAndProtocolVlanId;
  x_IFX_LLDP_VLAN_NAME xVlanName;
  x_IFX_LLDP_PROTOCOL_IDENTITY xProtocolIdentity;
  x_IFX_LLDP_MAC_PHY_CONFIG_STATUS xMacPhyConfigurationStatus;
  x_IFX_LLDP_MED_CAPABILITIES xLldpMedCapabilities;
  x_IFX_LLDP_NETWORK_POLICY xNetworkPolicy;
  x_IFX_LLDP_EXTENDED_POWER xExtendedPowerViaMdi;
}x_IFX_LLDP_TLV_CHOICE;

/* This interface structure is used to pass supported TLVs information to the 
   LLDP Agent */
typedef struct
{
  boolean bChassisId;
  boolean bPortId;
  boolean bSystemCapabilities;
  boolean bTimeToLive;
  boolean bPortVlanId;
  boolean bPortAndProtocolVlanId;
  boolean bVlanName;
  boolean bProtocolIdentity;
  boolean bMacPhyConfigurationStatus;
  boolean bLldpMedCapabilities;
  boolean bNetworkPolicy;
  boolean bExtendedPowerViaMdi;
  boolean bEndOfLldpdu;
}x_IFX_LLDP_TLV_SUPPORT;

/* This interface structure is used to pass TLV values to the LLDP Agent */
typedef struct
{
  uchar8 ucChassisId[255];
  uchar8 ucPortId[255];
  x_IFX_LLDP_SYS_CAPABILITIES xSystemCapabilities;
  x_IFX_LLDP_PORT_VLAN_ID xPortVlanId;
  x_IFX_LLDP_PORT_AND_PROTOCOL_VLAN_ID xPortAndProtocolVlanId;
  x_IFX_LLDP_VLAN_NAME xVlanName;
  x_IFX_LLDP_PROTOCOL_IDENTITY xProtocolIdentity;
  x_IFX_LLDP_MAC_PHY_CONFIG_STATUS xMacPhyConfigurationStatus;
  x_IFX_LLDP_MED_CAPABILITIES xLldpMedCapabilities;
  x_IFX_LLDP_NETWORK_POLICY xNetworkPolicy;
  x_IFX_LLDP_EXTENDED_POWER xExtendedPowerViaMdi;
}x_IFX_LLDP_INIT_TLV_VALUES;

/* This structure is used to fetch a specific local TLV information from 
   LLDP Agent */
typedef struct
{
  /* Input variable - TLV to be fetched */
  e_IFX_LLDP_INTF_TLV uiNewTlv;
  /* Output variable - TLV data is returned in this member variable */
  x_IFX_LLDP_TLV_CHOICE xValues;
}x_IFX_LLDP_TLV_VALUES;

/* This interface structure is used to pass initialization parameters to 
   initialise the LLDP Agent */
typedef struct 
{
  /* Operation mode - NCD, EPD */
  uint32 uiOpMode;
  /* Number of ports to be supported */
  uchar8 ucNumPorts;
  /* Maximum number of remote MIB entries to be supported */
  uint32 uiSupportedMibEntries;
  /* Indicates whether the raw remote LLDP packet data needs to stored 
     or not */
  boolean bLldpduStorage;

  /* Callback function pointer */
  lldpMgr_intf_funcptr pFuncPtr;  
}x_IFX_LLDP_MODULE_INIT;

/**************************************************************
 * Interface APIs
 **************************************************************/

/* This API is used to Initialize the LLDP module.
 *
 * pxInit - Pointer to a structure containing the values required 
 * to initialise the LLDP Module.
 */
char8 IFX_LLDP_MODULE_INIT (x_IFX_LLDP_MODULE_INIT *pxInit);

/* Initialises a specific LLDP port (LAN or PC).
 * 
 * pIntfName - This parameter specifies the Interface name to which this port
 *             should be associated.
 * ePort - An enumerator which specifies the port name i.e. PC or LAN.
 * uiFastStartRepeatCount - Specifies the default fast start repeat count value.
 * pxCfgData - Specifies default values for all local configuration parameters.
 * xTlvSupport - Specifies the TLVs to be supported on this port.
 * pxTlvValues - Pointer to a structure containing default values of supported 
 *               TLVs.
 */
char8 IFX_LLDP_PORT_INIT (uchar8 *pIntfName,
                          e_IFX_LLDP_PORT_NAME ePort,
                          x_IFX_LLDP_CONFIGCFG* pxCfgData,
                          x_IFX_LLDP_TLV_SUPPORT xTlvSupport,
                          x_IFX_LLDP_INIT_TLV_VALUES *pxTlvValues);

/* Enables an initialised port for transmission/reception of LLDP packets.
 *
 * ePort - Port Id to be enabled (PC or LAN)
 */
char8 IFX_LLDP_ENABLE_PORT (e_IFX_LLDP_PORT_NAME ePort);

/* Disables an enabled port.
 *
 * ePort - Port Id to be disabled (PC or LAN)
 */
char8 IFX_LLDP_DISABLE_PORT (e_IFX_LLDP_PORT_NAME ePort);

/* Adds a new TLV. Note that if the TLV which is being added is already
 * present, this interface returns error.
 *
 * ePort - Port to which the new TLV should be added.
 * pxTlvValues - New TLV information.
 */
char8 IFX_LLDP_TLV_ADD (e_IFX_LLDP_PORT_NAME ePort,
                        x_IFX_LLDP_TLV_VALUES *pxTlvValues);

/* Deletes an existing TLV. Note that if the TLV which is being deleted is not 
 * present, this interface returns error.
 *
 * ePort - Port from which the new TLV should be deleted.
 * eTlvType - TLV to be deleted.
 */
char8 IFX_LLDP_TLV_DEL (e_IFX_LLDP_PORT_NAME ePort,
                        e_IFX_LLDP_INTF_TLV eTlvType);

/* Modifies an existing TLV. Note that if the TLV which is being modified 
 * is not present, this interface returns error.
 *
 * ePort - Port at which the new TLV should be modified.
 * pxTlvValues - New TLV information.
 */
char8 IFX_LLDP_TLV_CHANGE (e_IFX_LLDP_PORT_NAME ePort,
                           x_IFX_LLDP_TLV_VALUES *pxTlvValues);

/* This API is used to extend the remote MIB when MIB full indication is 
 * received from LLDP Agent.
 *
 * uiNumber - Number by which the remote MIB should be extended.
 */
char8 IFX_LLDP_REMOTE_DB_EXTEND (uint32 uiNumber);

/* Deletes an entry from remote MIB.
 *
 * xMacAddres - MAC address of the entry to be deleted.
 */
char8 IFX_LLDP_REMOTE_DB_ENTRY_DEL (x_IFX_LLDP_MAC xMacAddres);

/* Fetches the raw PDU data from the remote MIB.
 *
 * xMacAddres - MAC address of the entry.
 * pPtr - The raw packet is returned in this parameter.
 */
char8 IFX_LLDP_PDU_REQUEST (x_IFX_LLDP_MAC xMacAddress,
                            void *pPtr);

/* Fetches the structured remote LLDP data from the remote MIB.
 *
 * xMacAddres - MAC address of the entry.
 * pxTlvValues - The TLV data is returned in this parameter.
 */

char8 IFX_LLDP_MIB_REQUEST (x_IFX_LLDP_MAC xMacAddress,
                            x_IFX_LLDP_TLV_VALUES *pxTlvValues);

/* Updates local configuration parameters.
 * 
 * ePort - Port for which the local configuration data is being updated.
 * pxCfgData - New local configuration data.
 */
char8 IFX_LLDP_LOCAL_CONFIG_UPDATED(e_IFX_LLDP_PORT_NAME ePort,
                                    x_IFX_LLDP_CONFIGCFG* pxCfgData);

/* Fetches the statistical data from LLDP Agent.
 *
 * ePort - Port for which the stats are to be fetched.
 * pxStats - Port status are returned in this output parameter.
 */
char8 IFX_LLDP_COUNTERS_REQUEST (e_IFX_LLDP_PORT_NAME ePort,
                                 x_IFX_LLDP_PORT_STATS* pxStats);

/* Shuts down the LLDP Agent. */
char8 IFX_LLDP_SHUTDOWN();

#endif /* __IFX_LLDP_INTF_H__ */
